"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Validate = void 0;
const constants_1 = require("../constants/constants");
const schemaPropertyName = constants_1.validationSchemaPropertyName;
function checkAndCreateSchemaProperty(target) {
    if (!target.hasOwnProperty(schemaPropertyName)) {
        Object.defineProperty(target, schemaPropertyName, {
            value: {}
        });
    }
}
/**
 * Validates a particular property using the constraints provided.
 */
function Validate(param) {
    return (target, propertyKey) => {
        // Check if validationSchema object is defined, if not, define it.
        checkAndCreateSchemaProperty(target);
        const schemaObject = target[schemaPropertyName];
        // Check if any such key already exists on the schema object.
        if (schemaObject.hasOwnProperty(propertyKey)) {
            throw new Error(`Model already contains a key with the name: ${propertyKey}`);
        }
        if (param.model) {
            const obj = new param.model();
            if (param.constraint.type === 'array') {
                if (!param.arrayElementsConstraint) {
                    throw new Error('Property "arrayElementConstraint" must be provided in case of array of objects.');
                }
                schemaObject[propertyKey] = param.constraint.items(param.arrayElementsConstraint.append(obj[schemaPropertyName]));
            }
            else {
                schemaObject[propertyKey] = param.constraint.append(obj[schemaPropertyName]);
            }
        }
        else {
            schemaObject[propertyKey] = param.constraint;
        }
    };
}
exports.Validate = Validate;
